use DBA_Reports

CREATE TABLE [dbo].[ServerList](
	[Server] [varchar](128) NOT NULL,
	[Server_email] [varchar](50) NULL,
 CONSTRAINT [PK_ServerList] PRIMARY KEY CLUSTERED 
(
	[Server] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 99) ON [PRIMARY]
) 
ON [PRIMARY]

-------------------------------------------------------------------------------------------------------------------------------

This job will populate the ServerList_Stage and ServerList tables.  Make sure you change <ADD YOUR INFO HERE> to your information. After you create this job, schedule it to run everyday.  I have mine running 3 times a day because I have users in multiple timezones as well as logging in at all differnet times.
USE [msdb]
GO

BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'DBA Monitoring' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'DBA Monitoring'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'DBA_Reports - Get New Servers', 
		@enabled=1, 
		@notify_level_eventlog=2, 
		@notify_level_email=2, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'Gets the new servers that are not in the table and sends out an email for new servers', 
		@category_name=N'DBA Monitoring', 
		@owner_login_name=N'sa', 
		@notify_email_operator_name=N'<ADD YOUR INFO HERE>', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'step1', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'USE [DBA_Reports]
GO

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N''[dbo].[ServerList_Stage]'') AND type in (N''U''))
DROP TABLE [dbo].[ServerList_Stage]
GO

USE [DBA_Reports]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO
CREATE TABLE [dbo].[ServerList_Stage](
	[Server] [varchar](128) NOT NULL,
	
 CONSTRAINT [PK_ServerList_Stage] PRIMARY KEY CLUSTERED 
(
	[Server] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 99) ON [PRIMARY]
) ON [PRIMARY]

GO

SET ANSI_PADDING OFF
GO

-- Load table with server names
-- This will only work if XP_CMDSHELL is Enabled

USE DBA_Reports
GO
Create table #Server ( [Server] [varchar](128) )
Insert Into #Server  
    Exec xp_cmdshell ''sqlcmd /Lc'' 
Insert Into ServerList_Stage  ([Server])
    select [Server] from #Server where [Server] is not null
DROP Table #Server
GO

--Send email to with new servers found

SET nocount ON
--
DECLARE @Subject VARCHAR (100)
SET @Subject=''SQL Server - New Servers Found Today''

DECLARE @Count AS INT
SELECT @Count=COUNT(*) FROM DBA_Reports.dbo.ServerList_Stage where [Server] not in (select [Server] from [ServerList])
PRINT @Count

IF @Count > 0
BEGIN

DECLARE @tableHTML NVARCHAR(MAX) ;
SET @tableHTML =
N''<table border="1">'' +
N''<tr>'' +
N''<th>Server</th>'' +
N''</tr>'' +
CAST ( ( SELECT td=[server],''''
FROM [ServerList_Stage]
where [Server] not in (select [Server] from [ServerList])

FOR XML PATH(''tr''), TYPE
) AS NVARCHAR(MAX) ) +
N''</table>'' ;

EXEC msdb.dbo.sp_send_dbmail
@profile_name = ''<ADD YOUR INFO HERE>'',
@recipients = ''<ADD YOUR INFO HERE>'',
@subject = @Subject,
@body = @tableHTML,
@body_format = ''HTML'' ;

END

INSERT INTO [DBA_Reports].[dbo].[ServerList]
           ([Server])
  SELECT  [Server]     
  FROM [DBA_Reports].[dbo].[ServerList_Stage]
     where [Server] not in (select [Server] from [DBA_Reports].[dbo].[ServerList])
GO

', 
		@database_name=N'DBA_Reports', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'Daily 3 times', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=8, 
		@freq_subday_interval=3, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20110512, 
		@active_end_date=99991231, 
		@active_start_time=63000, 
		@active_end_time=220000 
		
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO



CREATE TABLE [dbo].[Fragmented_jobStartStopTime](
	[ServerName] [varchar](50) NOT NULL,
	[StartJobTime] [datetime] NULL,
	[FinishJobTime] [datetime] NULL,
	[latest] [int] NULL
) ON [PRIMARY]



CREATE TABLE [dbo].[Fragmented_reorg_queries](
	[ServerName] [sysname] NULL,
	[DatabaseName] [sysname] NULL,
	[query] [varchar](537) NULL
) ON [PRIMARY]



CREATE TABLE [dbo].[Fragmented_reorg_queries_history](
	[ServerName] [sysname] NULL,
	[DatabaseName] [sysname] NULL,
	[LastChecked] [datetime] NOT NULL,
	[Query] [nchar](1000) NULL
) ON [PRIMARY]



CREATE TABLE [dbo].[FragmentedTables](
	[ServerName] [sysname] NULL,
	[DatabaseName] [sysname] NULL,
	[ObjectName] [char](255) NULL,
	[ObjectId] [int] NULL,
	[IndexName] [char](255) NULL,
	[IndexId] [int] NULL,
	[Lvl] [int] NULL,
	[CountPages] [int] NULL,
	[CountRows] [int] NULL,
	[MinRecSize] [int] NULL,
	[MaxRecSize] [int] NULL,
	[AvgRecSize] [int] NULL,
	[ForRecCount] [int] NULL,
	[Extents] [int] NULL,
	[ExtentSwitches] [int] NULL,
	[AvgFreeBytes] [int] NULL,
	[AvgPageDensity] [int] NULL,
	[ScanDensity] [decimal](18, 0) NULL,
	[BestCount] [int] NULL,
	[ActualCount] [int] NULL,
	[LogicalFrag] [decimal](18, 0) NULL,
	[ExtentFrag] [decimal](18, 0) NULL
) ON [PRIMARY]



CREATE TABLE [dbo].[FragmentedTablesAfterDefragRun](
	[ServerName] [sysname] NULL,
	[DatabaseName] [sysname] NULL,
	[ObjectName] [char](255) NULL,
	[ObjectId] [int] NULL,
	[IndexName] [char](255) NULL,
	[IndexId] [int] NULL,
	[Lvl] [int] NULL,
	[CountPages] [int] NULL,
	[CountRows] [int] NULL,
	[MinRecSize] [int] NULL,
	[MaxRecSize] [int] NULL,
	[AvgRecSize] [int] NULL,
	[ForRecCount] [int] NULL,
	[Extents] [int] NULL,
	[ExtentSwitches] [int] NULL,
	[AvgFreeBytes] [int] NULL,
	[AvgPageDensity] [int] NULL,
	[ScanDensity] [decimal](18, 0) NULL,
	[BestCount] [int] NULL,
	[ActualCount] [int] NULL,
	[LogicalFrag] [decimal](18, 0) NULL,
	[ExtentFrag] [decimal](18, 0) NULL
) ON [PRIMARY]



USE [DBA_Reports]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO
-- ==========================================================================
-- Author: Kimberly Killian
-- Create date: 5/19/11
-- Description:	Get the fragmented index's in each server/database (laptops)
-- ==========================================================================

CREATE procedure [dbo].[SelectAllServersFragmentedTables]
	@p_FragmentedLimit decimal = 5.0
as
	declare @err    int,
		@rc		    int, 
		@cmd        varchar(8000),
		@cmd2       varchar(8000),
		@ServerName sysname,
		@cmdmail	varchar(50),
		@cmdSprocDefrag varchar (50)
	
	set nocount on

IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[ServerList_frag]') AND type in (N'U'))
DROP TABLE [dbo].[ServerList_frag]

CREATE TABLE [dbo].[ServerList_frag](
	[Server] [varchar](128) NOT NULL,
	 CONSTRAINT [PK_ServerList_frag] PRIMARY KEY CLUSTERED 
(	[Server] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 99) ON [PRIMARY]
) ON [PRIMARY]

-- Load table with server names
-- This will only work if XP_CMDSHELL is Enabled

Create table #Server ( [Server] [varchar](128) )
Insert Into #Server  
    Exec xp_cmdshell 'sqlcmd /Lc' 
Insert Into ServerList_frag  ([Server])
    select [Server] from #Server --where SERVER in (select [SERVER] from dbo.ServerList where 
												  --[SERVER] not in (OPTIONAL list production main servers here))
DROP Table #Server 

---------------------------------------------------------------------------------------------------------
--	 Truncate the central table with the fragmentation information     
---------------------------------------------------------------------------------------------------------
TRUNCATE TABLE Fragmented_reorg_queries

---------------------------------------------------------------------------------------------------------
-- Loop on Server list and find fragmented tables  where they haven't cleaned up for at least 7 days
---------------------------------------------------------------------------------------------------------

	declare CursDefrag cursor for 
	
	--only run on servers that have not been checked in at least 7 days
	select Server from dbo.ServerList_frag where SERVER not in 
   (select distinct ServerName from dbo.Fragmented_reorg_queries_history group by servername having max(LastChecked) >= getdate()-7) 
     
	open CursDefrag
	fetch CursDefrag into @ServerName
 	 
	while @@FETCH_STATUS = 0
	BEGIN
	
	--send warning email to remote servers
	 select @cmdmail = 'sendemail_do_not_disconnect ' +  @ServerName
	 exec (@cmdmail)
	 
        /*Create a table in tempdb that will be populated at the remote
          server and later be accessed from the central database
          Collect the pre-fragmentation levels*/
             
		select @cmd = 'if exists (select 1 from ' + @servername + '.tempdb.dbo.sysobjects ' +
                            'where type = ''U'' AND NAME = ''fraglist'') ' +
                            'drop table tempdb.dbo.fraglist;' +
							
			'CREATE TABLE tempdb.dbo.fraglist (' +
		                           'ServerName sysname,' +
		                           'DatabaseName sysname,' +
		                           'ObjectName CHAR (255),' +
		                           'ObjectId INT,' +
		                           'IndexName CHAR (255),' +
		                           'IndexId INT,' +
		                           'Lvl INT,' +
		                           'CountPages INT,' +
		                           'CountRows INT, ' +
		                           'MinRecSize INT,' +
		                           'MaxRecSize INT,' +
		                           'AvgRecSize INT,' +
		                           'ForRecCount INT,' +
		                           'Extents INT,' +
		                           'ExtentSwitches INT,' +
		                           'AvgFreeBytes INT,' +
		                           'AvgPageDensity INT,' +
		                           'ScanDensity DECIMAL,' +
		                           'BestCount INT,' +
		                           'ActualCount INT,' +
		                           'LogicalFrag DECIMAL,' +
		                           'ExtentFrag DECIMAL) '
		              
		select @cmd2 = 'sqlcmd -E -S"' + @ServerName + '" -Q "' + 
                            @cmd + '"' 
		exec master..xp_cmdshell @cmd2

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR Creating the table in tempdb for SERVER - '+
                           @ServerName
			return -1
		end
     /*Query the fragmenation data into a table on tempdb for each remote server
        Insert fragmentation information into a temporary table.
        Insert fragmentation information including server name and database name into 
        the remote table created in tempdb. 
        Create defrag/reindex queries and write to a table to be used later.
        Write defrag/reindex queries to a history table to be used later*/
        

		select @cmd = 
		'exec [' + @ServerName + '].master.dbo.sp_msforeachdb ' +
		'''use [?]; print ''''?''''; 
             if ''''?'''' in (''''tempdb'''',''''msdb'''',''''master'''',
                              ''''model'''',''''Northwind5'''',''''pubs'''')
                        return
		create table #t (cmd varchar(1000)); 
		insert into #t select 
                     ''''dbcc showcontig ('''''''''''' + TABLE_SCHEMA + 
                     ''''.'''' + TABLE_NAME + '''''''''''') 
                     WITH TABLERESULTS, ALL_INDEXES'''' 
             from INFORMATION_SCHEMA.TABLES 
		where TABLE_TYPE = ''''BASE TABLE''''; 
		declare @cmd varchar(1000) 
		declare cursRemoteCheck cursor for select cmd from #t
		open cursRemoteCheck
		fetch cursRemoteCheck into @cmd
		while @@fetch_status = 0
		begin
		      select ObjectName, ObjectId, IndexName, IndexId, Lvl, 
                           CountPages, CountRows, MinRecSize, MaxRecSize, 
                           AvgRecSize, ForRecCount, Extents, ExtentSwitches,
                           AvgFreeBytes, AvgPageDensity, ScanDensity, 
                           BestCount, ActualCount, LogicalFrag, ExtentFrag
		       into #fraglist from tempdb.dbo.fraglist where 1=2
			insert into #fraglist exec (@cmd)
			insert into tempdb.dbo.fraglist 
                          select ''''' +  @ServerName + ''''', 
                                 db_name(),* from #fraglist 
                          WHERE IndexID not in (0,255)
                            And LogicalFrag > ''''' + 
                                convert(varchar(10),@p_FragmentedLimit) +
                                '''''
		     drop table #fraglist
			fetch cursRemoteCheck into @cmd
		end
		close cursRemoteCheck
		deallocate cursRemoteCheck
		drop table #t'''

		exec (@cmd)

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR Collecting fragmentation information on:  '' + @ServerName
			return -1
		end

		-- Insert the fragmentation details to the remote server table
		select @cmd = 'insert into FragmentedTables select * from ['+
                    @ServerName + '].tempdb.dbo.fraglist'
		exec (@cmd)

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR selecting data from remote table on: ' ' + @ServerName
			return -1
		end

--get the fragmented tables and generate scripts  
 --save the history
 
			select @cmd = 'INSERT INTO [DBA_Reports].[dbo].[Fragmented_reorg_queries]
						  ([ServerName]   ,[DatabaseName]  ,[query]) 
							 SELECT ServerName,  DatabaseName,
							 ''ALTER INDEX'' + '' '' + RTRIM(IndexName) + '' '' + ''ON '' + RTRIM([ObjectName]) + '' '' + ''REORGANIZE'' as query
							 FROM [DBA_Reports].[dbo].[FragmentedTables]
							 where LogicalFrag between 5 and 30
			 	
			 INSERT INTO [DBA_Reports].[dbo].[Fragmented_reorg_queries]
					     ([ServerName]  ,[DatabaseName]  ,[query])
							 SELECT ServerName,  DatabaseName,
							 ''ALTER INDEX'' + '' '' + RTRIM(IndexName) + '' '' + ''ON '' + RTRIM([ObjectName]) + '' '' + ''REBUILD'' as query
							 FROM [DBA_Reports].[dbo].[FragmentedTables]
							 where LogicalFrag >= 31			 

			 INSERT INTO [DBA_Reports].[dbo].[Fragmented_reorg_queries_history]
					   ([ServerName] ,[DatabaseName] , [Query], [LastChecked])
						SELECT [ServerName], [DatabaseName], [query], GETDATE() as LastChecked
						FROM [DBA_Reports].[dbo].[Fragmented_reorg_queries]'
		exec (@cmd)
		
		--Run the queries generated for each server on each server 
		select @cmdSprocDefrag = 'defrag_Fragmented_tables ' + @ServerName 
		exec(@cmdSprocDefrag)

		-- Drop the remote table in tempdb           
		select @cmd = 
                  'if exists (select 1 from tempdb.dbo.sysobjects ' + 
                              'where type = ''U'' AND NAME = ''fraglist'') ' +
                                 'drop table tempdb.dbo.fraglist' 
		select @cmd2 = 'sqlcmd -E -S"' + @ServerName + '" -Q "' + @cmd +
                            '"' 
		exec master..xp_cmdshell @cmd2

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR unable to drop remote table on: ' + @ServerName
			return -1
		end
	
	--run query to check level of fragmentation after defrag
	--same as above
		select @cmd = 'if exists (select 1 from tempdb.dbo.sysobjects ' +
                            'where type = ''U'' AND NAME = ''fraglistafter'') ' +
                            'drop table tempdb.dbo.fraglistafter;' +
		               
			'CREATE TABLE tempdb.dbo.fraglistafter (' +
		                           'ServerName sysname,' +
		                           'DatabaseName sysname,' +
		                           'ObjectName CHAR (255),' +
		                           'ObjectId INT,' +
		                           'IndexName CHAR (255),' +
		                           'IndexId INT,' +
		                           'Lvl INT,' +
		                           'CountPages INT,' +
		                           'CountRows INT, ' +
		                           'MinRecSize INT,' +
		                           'MaxRecSize INT,' +
		                           'AvgRecSize INT,' +
		                           'ForRecCount INT,' +
		                           'Extents INT,' +
		                           'ExtentSwitches INT,' +
		                           'AvgFreeBytes INT,' +
		                           'AvgPageDensity INT,' +
		                           'ScanDensity DECIMAL,' +
		                           'BestCount INT,' +
		                           'ActualCount INT,' +
		                           'LogicalFrag DECIMAL,' +
		                           'ExtentFrag DECIMAL) '
		              
		select @cmd2 = 'sqlcmd -E -S"' + @ServerName + '" -Q "' + 
                            @cmd + '"' 
		exec master..xp_cmdshell @cmd2

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR unable to create table in tempdb on:  ' + @ServerName
			return -1
		end

	    /*Query the fragmenation data into a table on tempdb for each remote server
        Insert fragmentation information into a temporary table.
        Insert fragmentation information including server name and database name into 
        the remote table created in tempdb. */
     
		select @cmd = 
		'exec [' + @ServerName + '].master.dbo.sp_msforeachdb ' +
		'''use [?]; print ''''?''''; 
             if ''''?'''' in (''''tempdb'''',''''msdb'''',''''master'''',
                              ''''model'''',''''Northwind5'''',''''pubs'''')
                        return
		create table #a (cmd varchar(1000)); 
		insert into #a select 
                     ''''dbcc showcontig ('''''''''''' + TABLE_SCHEMA + 
                     ''''.'''' + TABLE_NAME + '''''''''''') 
                     WITH TABLERESULTS, ALL_INDEXES'''' 
             from INFORMATION_SCHEMA.TABLES 
		where TABLE_TYPE = ''''BASE TABLE''''; 
		declare @cmd varchar(1000) 
		declare cursRemoteCheckafter cursor for select cmd from #a
		open cursRemoteCheckafter
		fetch cursRemoteCheckafter into @cmd
		while @@fetch_status = 0
		begin
		       select ObjectName, ObjectId, IndexName, IndexId, Lvl, 
                           CountPages, CountRows, MinRecSize, MaxRecSize, 
                           AvgRecSize, ForRecCount, Extents, ExtentSwitches,
                           AvgFreeBytes, AvgPageDensity, ScanDensity, 
                           BestCount, ActualCount, LogicalFrag, ExtentFrag
		       into #fraglistafter from tempdb.dbo.fraglistafter where 1=2
			insert into #fraglistafter exec (@cmd)
			insert into tempdb.dbo.fraglistafter 
                          select ''''' +  @ServerName + ''''', 
                                 db_name(),* from #fraglistafter 
                          WHERE IndexID not in (0,255)
                            And LogicalFrag > ''''' + 
                                convert(varchar(10),@p_FragmentedLimit) +
                                '''''
		       drop table #fraglistafter
			fetch cursRemoteCheckafter into @cmd
		end
		close cursRemoteCheckafter
		deallocate cursRemoteCheckafter
		drop table #a'''

		exec (@cmd)

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR Collecting fragmentation information on:  ' + @ServerName
			return -1
		end

		-- Insert the fragmentation details to the remote server table
		select @cmd = 'insert into FragmentedTablesAfterDefragRun select * from ['+
                    @ServerName + '].tempdb.dbo.fraglistafter'
		exec (@cmd)

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR selecting data from remote table on:' + @ServerName
			return -1
		end
		
		-- Drop the remote table in tempdb         
		select @cmd = 
                  'if exists (select 1 from tempdb.dbo.sysobjects ' + 
                              'where type = ''U'' AND NAME = ''fraglistafter'') ' +
                                 'drop table tempdb.dbo.fraglistafter' 
		select @cmd2 = 'sqlcmd -E -S"' + @ServerName + '" -Q "' + @cmd +
                            '"' 
		exec master..xp_cmdshell @cmd2

		select @err = @@error
		IF @err <> 0 
		begin
			select 'ERROR dropping remote table on: ' + @ServerName
			return -1
		end

	
	--send completion email to remote servers
		 select @cmdmail = 'sendemail_All_Clear ' +  @ServerName
		 exec (@cmdmail)
		 		 
		fetch CursDefrag into @ServerName
	END
	close CursDefrag
	deallocate CursDefrag

GO



USE [DBA_Reports]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ======================================================================
-- Author: Kimberly Killian
-- Create date: 5/19/11
-- Description:	Kick off the Defrag statements for each server/database
-- =========================================================================
CREATE PROCEDURE [dbo].[defrag_Fragmented_tables]
	@ServerName varchar(50)
AS

 SET NOCOUNT ON;
 Declare @query varchar(8000)
 Declare @DatabaseName varchar(128) 
 
 Declare query_cursor CURSOR FORWARD_ONLY FOR
 select query + ';' from dbo.Fragmented_reorg_queries where [ServerName] = @ServerName
 
 Open query_cursor
 fetch Next from query_cursor into @query
 While @@FETCH_STATUS = 0
 
 Begin
 
 select @DatabaseName = databasename from dbo.Fragmented_reorg_queries where [ServerName] = @ServerName
 --use sqlcmd to run each query on each server
  
 Declare @OpenServerRunQuery varchar(8000)
 SET @OpenServerRunQuery =  'sqlcmd -E -S "' + @ServerName + '" -d "' + @DatabaseName + '" -q "' + @query + '"'
 
 --exec queries for each server
 exec master..xp_cmdshell @OpenServerRunQuery
 
fetch Next from query_cursor into @query

END

--clean up
close query_cursor
deallocate query_cursor

GO


USE [DBA_Reports]
GO
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Kimberly Killian
-- Create date: 5/19/11
-- Description:	send email to laptop users not to disconnect
-- =============================================
CREATE PROCEDURE [dbo].[sendemail_do_not_disconnect] 
		@ServerName varchar(50)
AS
BEGIN
	SET NOCOUNT ON;
 Declare @email varchar(25)
 select @email = Server_email from dbo.ServerList where Server = @ServerName

--log the start time of the job for this server
INSERT INTO [Fragmented_jobStartStopTime]
            ([ServerName],[StartJobTime], [latest])
     VALUES (@ServerName, GETDATE(), 0);

--send the email

 Declare @body varchar (1000)
 SET @body = '<html><body><font face=verdana size=2>                                                             
  <p><strong>Database Maintenance has started on ' + @ServerName + '.  Please do not disconnect your machine from the network.  This may take some time. </strong><br>
  An email will be sent upon completion.</p>
  <p><i>We apologize for any inconvenience this may cause.  Please contact me if you experience any issues or need to disconnect.</i></p>                                
  <p>Thank you,<br><br>                               
  <strong><ADD YOUR INFO(name) HERE><br>                              
  <ADD YOUR INFO (title) HERE><br> 
  <ADD YOUR INFO (company) HERE> </strong><br>
  <ADD YOUR INFO (phone) HERE><br/>                                
  <ADD YOUR INFO (email) HERE>
  </p>                                
  </body></html>'
 
   --send warning email do not remove from network			        
	
	EXEC msdb.dbo.sp_send_dbmail
				@profile_name	= 'ADD YOUR INFO HERE',
				@recipients		=@email,
				@copy_recipients = 'ADD YOUR INFO (title) HERE',
				@subject		='AUTOMATED SEND: Database Maintenance Started Please Do not disconnect',
				@body			=@body,
				@body_format	='HTML';		
                   
END

GO


USE [DBA_Reports]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:  Kimberly Killian
-- Create date: 5/19/11
-- Description:	send all clear email to laptop users
-- =============================================
CREATE PROCEDURE [dbo].[sendemail_All_Clear] 
	@ServerName varchar(50)
AS
BEGIN
	SET NOCOUNT ON;
 Declare @email varchar(25)
 select @email = Server_email from dbo.ServerList where Server = @ServerName

--set the finsih job time 
UPDATE [DBA_Reports].[dbo].[Fragmented_jobStartStopTime]
SET [FinishJobTime] = GETDATE(), [latest] = 1
where servername = @ServerName and latest = 0;

 Declare @body varchar (1000)
 SET @body = '<html><body><font face=verdana size=2>                                                             
  <p><strong>Database Maintenance has completed on ' + @ServerName + '. </strong><br></p>
  <p><i>Thank you for your time and patience.</i></p>                                
                               
  <strong><ADD YOUR INFO(name) HERE><br>                              
  <ADD YOUR INFO (title) HERE><br> 
  <ADD YOUR INFO (company) HERE> </strong><br>
  <ADD YOUR INFO (phone) HERE><br/>                                
  <ADD YOUR INFO (email) HERE>
  </p>                                
  </body></html>'
 
   --send all clear email do not remove from network			        
	
	EXEC msdb.dbo.sp_send_dbmail
				@profile_name	= '<ADD YOUR INFO HERE>',
				@recipients		=@email,
				@copy_recipients = '<ADD YOUR INFO HERE>',
				@subject		='AUTOMATED SEND: Database Maintenance has completed',
				@body			=@body,
				@body_format	='HTML';		
                  
END

GO


USE [msdb]
GO

BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0
IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'DBA Monitoring' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'DBA Monitoring'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'DBA_Reports - Laptop Fragmentation', 
		@enabled=1, 
		@notify_level_eventlog=2, 
		@notify_level_email=2, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'Notifies laptop user that db maintenance will begin
Checks level of fragmentation on each laptop
Creates scripts to either redinex or reorganize
Send mail with list of laptops with fragmentation
Send mail with scripts to run
Notifies users that db maintenance is completed
Sends admin email notifying who was defragged today
Sends admin email notifying who has not defragged for at least 2 weeks
Send admin email on which indexs were defragged', 
		@category_name=N'DBA Monitoring', 
		@owner_login_name=N'sa', 
		@notify_email_operator_name=N'<ADD YOUR INFO HERE>', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [Call Fragmentation SPROC]    Script Date: 06/14/2011 13:36:03 ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Call Fragmentation SPROC', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'exec SelectAllServersFragmentedTables



', 
		@database_name=N'DBA_Reports', 
		@output_file_name=N'<ADD YOUR INFO HERE>', 
		@flags=4
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Send email of who was defragged today', 
		@step_id=2, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'EXEC msdb.dbo.sp_start_job ''DBA_Reports - Report Laptops Defragged Today''', 
		@database_name=N'DBA_Reports', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [send email for Laptops not Defragged for at least 2 weeks]    Script Date: 06/14/2011 13:36:04 ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'send email for Servers not Defragged for at least 2 weeks', 
		@step_id=3, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'EXEC msdb.dbo.sp_start_job ''DBA_Reports - Report Servers not Defragged for at least 2 weeks''', 
		@database_name=N'DBA_Reports', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'email what was defragged and initial numbers', 
		@step_id=4, 
		@cmdexec_success_code=0, 
		@on_success_action=3, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'SET nocount ON
--
DECLARE @Subject VARCHAR (100)
SET @Subject=''SQL Server - Indexs on Servers Defragged Today''

DECLARE @Count AS INT
SELECT distinct @Count=COUNT(ServerName) FROM [DBA_Reports].[dbo].[FragmentedTables]
  
PRINT @Count

IF @Count > 0
BEGIN

DECLARE @tableHTML NVARCHAR(MAX) ;
SET @tableHTML =
N''<table border="1">'' +
N''<tr>'' +
N''<th>Server Name</th>'' +
N''<th>Database Name</th>'' +
N''<th>Object Name</th>'' +
N''<th>Index Name</th>'' +
N''<th>Pages</th>'' +
N''<th>Fragmentation % Before</th>'' +
N''<th>Fragmentation % After</th>''+
N''</tr>'' +
CAST ( (SELECT td= dbo.FragmentedTablesAfterDefragRun.ServerName,''''
,td= dbo.FragmentedTablesAfterDefragRun.DatabaseName,''''
,td= dbo.FragmentedTablesAfterDefragRun.ObjectName, ''''
,td= dbo.FragmentedTablesAfterDefragRun.IndexName ,''''
,td= dbo.FragmentedTablesAfterDefragRun.CountPages, ''''
,td= dbo.FragmentedTables.LogicalFrag, ''''
,td= dbo.FragmentedTablesAfterDefragRun.LogicalFrag,''''

FROM dbo.FragmentedTables INNER JOIN dbo.FragmentedTablesAfterDefragRun 
ON  (dbo.FragmentedTablesAfterDefragRun.ObjectName = dbo.FragmentedTables.ObjectName) 
AND (dbo.FragmentedTablesAfterDefragRun.DatabaseName = dbo.FragmentedTables.DatabaseName) 
AND (dbo.FragmentedTables.ServerName = dbo.FragmentedTablesAfterDefragRun.ServerName)
AND (dbo.FragmentedTablesAfterDefragRun.IndexName = dbo.FragmentedTables.IndexName)
ORDER BY dbo.FragmentedTablesAfterDefragRun.ServerName,  dbo.FragmentedTablesAfterDefragRun.DatabaseName,  dbo.FragmentedTables.LogicalFrag,  dbo.FragmentedTablesAfterDefragRun.IndexName
FOR XML PATH(''tr''), TYPE
) AS NVARCHAR(MAX) ) +
N''</table>'' ;

EXEC msdb.dbo.sp_send_dbmail
@profile_name = ''<ADD YOUR INFO HERE>'',
@recipients = ''<ADD YOUR INFO HERE>'',
@subject = @Subject,
@body = @tableHTML,
@body_format = ''HTML'' ;

END


', 
		@database_name=N'DBA_Reports', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Clean Up Temp tables', 
		@step_id=5, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'TRUNCATE TABLE FragmentedTables

TRUNCATE Table dbo.FragmentedTablesAfterDefragRun', 
		@database_name=N'DBA_Reports, 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'schedule', 
		@enabled=1, 
		@freq_type=8, 
		@freq_interval=62, 
		@freq_subday_type=1, 
		@freq_subday_interval=0, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=1, 
		@active_start_date=20110523, 
		@active_end_date=99991231, 
		@active_start_time=140000, 
		@active_end_time=235959 
	
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO 


USE [msdb]
GO

BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'[Uncategorized (Local)]' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'[Uncategorized (Local)]'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'DBA_Reports - Report Laptops Defragged Today', 
		@enabled=1, 
		@notify_level_eventlog=2, 
		@notify_level_email=2, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'Lists out the servers that have been defragged today', 
		@category_name=N'[Uncategorized (Local)]', 
		@owner_login_name=N'sa', 
		@notify_email_operator_name=N'<ADD YOUR INFO HERE>', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'send mail', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'SET nocount ON
--
DECLARE @Subject VARCHAR (100)
SET @Subject=''SQL Server - Laptops Defragged Today''

DECLARE @Count AS INT
SELECT distinct @Count=COUNT(ServerName) FROM [DBA_Reports].[dbo].[Fragmented_reorg_queries_history]
  group by ServerName, DatabaseName
  having datepart(month, max([LastChecked])) = datepart(month, GETDATE())
  and datepart(day, max([LastChecked])) = datepart(day, GETDATE())
  and datepart(year, max([LastChecked])) = datepart(year, GETDATE())
  
PRINT @Count

IF @Count > 0
BEGIN

DECLARE @tableHTML NVARCHAR(MAX) ;
SET @tableHTML =
N''<table border="1">'' +
N''<tr>'' +
N''<th>ServerName</th>'' +
N''<th>DatabaseName</th>'' +
N''<th>Last Checked</th>'' +
N''<th>Run Duration</th>'' +
N''</tr>'' +
CAST ( (SELECT td=[Fragmented_reorg_queries_history].[servername],''''
,td=[databasename],''''
,td=max([lastchecked]),''''
,td=right(convert(varchar(30), (max([FinishJobTime]) - max([StartJobTime])),121),12),''''

FROM [Fragmented_reorg_queries_history] 
join [Fragmented_jobStartStopTime] on  Fragmented_reorg_queries_history.servername = Fragmented_jobStartStopTime.servername
group by dbo.Fragmented_reorg_queries_history.ServerName, DatabaseName
having datepart(month, max([LastChecked])) = datepart(month, GETDATE())
and datepart(day, max([LastChecked])) = datepart(day, GETDATE())
and datepart(year, max([LastChecked])) = datepart(year, GETDATE())
FOR XML PATH(''tr''), TYPE
) AS NVARCHAR(MAX) ) +
N''</table>'' ;

EXEC msdb.dbo.sp_send_dbmail
@profile_name = ''<ADD YOUR INFO HERE>'',
@recipients = ''<ADD YOUR INFO HERE>'',
@subject = @Subject,
@body = @tableHTML,
@body_format = ''HTML'' ;

END', 
		@database_name=N'DBA_Reports', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO


USE [msdb]
GO

BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0

IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'[Uncategorized (Local)]' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'[Uncategorized (Local)]'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'DBA_Reports - Report servers not Defragged for at least 2 weeks', 
		@enabled=1, 
		@notify_level_eventlog=2, 
		@notify_level_email=2, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'List of servers not defragged for at least 2 weeks', 
		@category_name=N'[Uncategorized (Local)]', 
		@owner_login_name=N'sa', 
		@notify_email_operator_name=N'<ADD YOUR INFO HERE>', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'send email', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'SET nocount ON
--
DECLARE @Subject VARCHAR (100)
SET @Subject=''DBA Reports - Servers not defragged in the last 2 weeks''

DECLARE @Count AS INT
SELECT distinct @Count=COUNT(ServerName) FROM [DBA_Reports].[dbo].[Fragmented_reorg_queries_history]
  group by ServerName, DatabaseName
  having max([LastChecked]) <=  GETDATE()-14
  
PRINT @Count

IF @Count > 0
BEGIN

DECLARE @tableHTML NVARCHAR(MAX) ;
SET @tableHTML =
N''<table border="1">'' +
N''<tr>'' +
N''<th>ServerName</th>'' +
N''<th>DatabaseName</th>'' +
N''<th>Last Checked</th>'' +
N''</tr>'' +
CAST ( ( SELECT td=[servername],''''
,td=[databasename],''''
,td=max([lastchecked]),''''
FROM [Fragmented_reorg_queries_history]
group by ServerName, DatabaseName
having max([LastChecked]) <=  GETDATE()-14
FOR XML PATH(''tr''), TYPE
) AS NVARCHAR(MAX) ) +
N''</table>'' ;

EXEC msdb.dbo.sp_send_dbmail
@profile_name = ''<ADD YOUR INFO HERE>'',
@recipients = ''<ADD YOUR INFO HERE>'',
@subject = @Subject,
@body = @tableHTML,
@body_format = ''HTML'' ;

END', 
		@database_name=N'master', 
		@flags=0
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO



